const express = require('express');
const dotenv = require('dotenv');
const path = require('path');
const { Eta } = require("eta");
const cookieParser = require('cookie-parser');
const bodyParser = require('body-parser');
const crypto = require('crypto');
const nodemailer  = require('nodemailer');
const app = express();
const authTokens = {};
// https://github.com/jkasun/stack-abuse-express-authentication

// load config
dotenv.config({path: './config/config.env'});
const users = require('./config/users');

const getHashedPassword = (password) => {
    const sha256 = crypto.createHash('sha256');
    const hash = sha256.update(password).digest('base64');
    return hash;
}

const generateAuthToken = () => {
    return crypto.randomBytes(30).toString('hex');
}

app.use(express.static(path.join(__dirname, 'public')));
// to support URL-encoded bodies
app.use(bodyParser.urlencoded({ extended: true }));
app.use(cookieParser());

app.use((req, res, next) => {
    const authToken = req.cookies['AuthToken'];
    req.user = authTokens[authToken];
    next();
});

// set the eta template engine
const eta = new Eta({
  // Views directory path
  views: path.join(__dirname, "views"), // 

  // Any other option...
  cache: true
});


// @desc Login Landing page
// Route: Get /
app.get("/", (req, res) => {
  const renderedTemplate = eta.render("home", { 
      title: "My New Site",
      message: "Welcome to My New Site!"
  }); // create `index.eta` in the `views` folder
  res.status(200).send(renderedTemplate)
});
  
app.get("/aboutus", (req, res) => {
  const renderedTemplate = eta.render("aboutus", { 
      title: "AboutUs"
  });
  res.status(200).send(renderedTemplate)
});
  
app.get("/findus", (req, res) => {
  const renderedTemplate = eta.render("findus", { 
      title: "FindUs"
  }); 
  res.status(200).send(renderedTemplate)
});
  
app.get("/contactus", (req, res) => {
  const renderedTemplate = eta.render("contact", { 
      title: "Contact Us"
  }); 
  res.status(200).send(renderedTemplate)
});
  
app.post('/contact', (req, res) => {

  // Instantiate the SMTP server
  const smtpTrans = nodemailer.createTransport({
    sendmail: true,
    newline: 'unix',
    path: '/usr/sbin/sendmail'
    });

  // Specify what the email will look like
  const mailOpts = {
    from: process.env.EMAILUSER, // This is ignored
    to: process.env.EMAILUSER,
    subject: 'New message from contact form at example.com',
    text: `${req.body.name} (${req.body.email}) says: ${req.body.message}`
  }

  // Attempt to send the email
  smtpTrans.sendMail(mailOpts, (error, response) => {
	if (req.body.token === '41') {
    if (error) {
      const renderedTemplate = eta.render("contact", { 
            message: 'Failure to Send Message',
            messageClass: 'alert-danger',
			title: 'Contact US'
  });
  res.status(200).send(renderedTemplate)
    } else {
      const renderedTemplate = eta.render("contact", { 
            message: 'Successfully Sent Message',
            messageClass: 'alert-danger',
			title: 'Contact US'
  });
  res.status(200).send(renderedTemplate)
  
  };
      } else {
      const renderedTemplate = eta.render("contact", { 
            message: 'Figure 41 not entered',
            messageClass: 'alert-danger',
			title: 'Contact US'
  });
  res.status(200).send(renderedTemplate)
  };
 });
 
});

app.get('/login', (req, res) => {
  const renderedTemplate = eta.render("login", { 
      title: "Login"
  });
  res.status(200).send(renderedTemplate)
});

app.post('/login', (req, res) => {
    const { email, password } = req.body;
    const hashedPassword = getHashedPassword(password);

    const user = users.find(u => {
        return u.email === email && hashedPassword === u.password
    });

    if (user) {
        const authToken = generateAuthToken();

        authTokens[authToken] = email;

        res.cookie('AuthToken', authToken);
        res.redirect('/protected');
        return;
    } else {
          const renderedTemplate = eta.render("login", { 
             message: 'Invalid username or password',
            messageClass: 'alert-danger',
            title: "Login"
  }); 
  res.status(200).send(renderedTemplate)
    };
});

app.get('/registerJoe2Morrow713pm', (req, res) => {
    if (req.user) {	
    const renderedTemplate = eta.render('private/register', { 
      title: 'Register'
      });
      res.status(200).send(renderedTemplate)		
		
    } else {
          const renderedTemplate = eta.render("login", { 
            message: 'Please login to continue',
            messageClass: 'alert-danger',
			title: 'Login'
        }); 
        res.status(200).send(renderedTemplate)        
    }
});

app.post('/registerJoe2Morrow713pm', (req, res) => {
if (req.user) {
    const { email, password, confirmPassword } = req.body;

    if (password === confirmPassword) {
        if (users.find(user => user.email === email)) {
          const renderedTemplate = eta.render('private/register', { 
                message: 'User already registered.',
                messageClass: 'alert-danger',
				title: 'Register'
           });
             res.status(200).send(renderedTemplate)	           

            return;
        }

        const hashedPassword = getHashedPassword(password);

        const renderedTemplate = eta.render('private/regsuccess', { 
            message: 'Now put email, hashedPassword, a first and last name in the app.js file in users.',
            messageClass: 'alert-success',
			title: 'Success',
            email: email,
            password: password,
            hashedPassword: hashedPassword
           });
             res.status(200).send(renderedTemplate)    
        
    } else {
        res.render('private/register', {
            message: 'Password does not match.',
            messageClass: 'alert-danger',
			title: 'Register'
        });
        
        const renderedTemplate = eta.render('private/register', { 
            message: 'Password does not match.',
            messageClass: 'alert-danger',
				title: 'Register'
           });
             res.status(200).send(renderedTemplate)	
          }
    } else {

   const renderedTemplate = eta.render("login", { 
        message: 'Please login first',
        messageClass: 'alert-danger',
            title: "Login"
  }); 
  res.status(200).send(renderedTemplate)
}
});

app.get('/protected', (req, res) => {
    if (req.user) {
	  const renderedTemplate = eta.render('private/priv1', { 
			title: 'Priv1'
        }); 
         res.status(200).send(renderedTemplate)
			
    } else {

      const renderedTemplate = eta.render("login", { 
            message: 'Please login to continue',
            messageClass: 'alert-danger',
			title: 'Login'
       });
       res.status(200).send(renderedTemplate)
    }
});

app.get('/priv2', (req, res) => {
    if (req.user) {
	  const renderedTemplate = eta.render('private/priv2', { 
			title: 'Priv2'
        }); 
         res.status(200).send(renderedTemplate)
			
    } else {

      const renderedTemplate = eta.render("login", { 
            message: 'Please login to continue',
            messageClass: 'alert-danger',
			title: 'Login'
       });
       res.status(200).send(renderedTemplate)
    }
});

app.get('/logout', (req, res) => {
    res.logout
    res.redirect('/')

});

// error page for non-existent endpoints
app.get('*', (req, res) => {
  const renderedTemplate = eta.render("none", { 
      title: "404"
  });
  res.status(200).send(renderedTemplate)
});
  
PORT = process.env.PORT || 5000; // one can override and set the port in the config.env file
app.listen(PORT);
